from django.contrib import admin
from rolodex.models import OpenRecordsLaw, PersonRole, P2P_Type, Org2Org_Type, P2Org_Type, Contact, Person, Org, OrgContactRole


class ContactInline(admin.StackedInline):
	model = Contact
	extra = 0


class POAdmin(admin.ModelAdmin):
	inlines = [ContactInline, ]

admin.site.register(OpenRecordsLaw)
admin.site.register(PersonRole)
admin.site.register(OrgContactRole)
admin.site.register(P2P_Type)
admin.site.register(Org2Org_Type)
admin.site.register(P2Org_Type)

admin.site.register(Person, POAdmin)
admin.site.register(Org, POAdmin)



from rest_framework import viewsets
from rolodex.API import serializers
from rolodex.models import Person, Org, Contact, PersonRole, OrgContactRole, P2P, Org2Org, P2Org, Org2P, P2P_Type, Org2Org_Type, P2Org_Type
from rest_framework.views import APIView
from rest_framework.response import Response
from django.shortcuts import get_object_or_404


class RoleViewSet(viewsets.ModelViewSet):
	queryset = PersonRole.objects.all()
	serializer_class = serializers.RoleSerializer


class ContactRoleViewSet(viewsets.ModelViewSet):
	queryset = OrgContactRole.objects.all()
	serializer_class = serializers.ContactRoleSerializer


class P2P_TypeViewSet(viewsets.ModelViewSet):
	queryset = P2P_Type.objects.all()
	serializer_class = serializers.P2P_TypeSerializer


class Org2Org_TypeViewSet(viewsets.ModelViewSet):
	queryset = Org2Org_Type.objects.all()
	serializer_class = serializers.Org2Org_TypeSerializer


class P2Org_TypeViewSet(viewsets.ModelViewSet):
	queryset = P2Org_Type.objects.all()
	serializer_class = serializers.P2Org_TypeSerializer


class PersonViewSet(viewsets.ModelViewSet):
	queryset = Person.objects.all().order_by('id')
	serializer_class = serializers.PersonSerializer


class OrgViewSet(viewsets.ModelViewSet):
	queryset = Org.objects.all().order_by('id')
	serializer_class = serializers.OrgSerializer


class ContactViewSet(viewsets.ModelViewSet):
	queryset = Contact.objects.all().order_by('id')
	serializer_class = serializers.ContactSerializer


class P2PViewSet(viewsets.ModelViewSet):
	queryset = P2P.objects.all().order_by('id')
	serializer_class = serializers.P2PSerializer


class Org2OrgViewSet(viewsets.ModelViewSet):
	queryset = Org2Org.objects.all().order_by('id')
	serializer_class = serializers.Org2OrgSerializer


class Org2PViewSet(viewsets.ModelViewSet):
	queryset = Org2P.objects.all().order_by('id')
	serializer_class = serializers.Org2PSerializer


class P2OrgViewSet(viewsets.ModelViewSet):
	queryset = P2Org.objects.all().order_by('id')
	serializer_class = serializers.P2OrgSerializer


class GetEmployees(APIView):
	model = Person
	queryset = Person.objects.all()
	'''
	Get all an orgs empoyees.
	'''
	def get(self, request, pk):
		org = get_object_or_404(Org, pk=pk)
		employees = org.get_employees()
		serializer = serializers.PersonSerializer(employees, many=True, context={'request': request})
		return Response(serializer.data)



#####################
# DATABASE SETTINGS #
#####################
import dj_database_url
from django.conf import settings
import os

if 'rolodex' not in settings.DATABASES:
	if 'ROLODEX_DB' in os.environ:
		settings.DATABASES['rolodex'] = dj_database_url.parse(os.environ.get('ROLODEX_DB'))
		settings.DATABASE_ROUTERS.append('rolodex.routers.RolodexRouter')
else:
	settings.DATABASE_ROUTERS.append('rolodex.routers.RolodexRouter')
#######################################################

if not hasattr(settings, 'ROLODEX_SECURE'):
	settings.ROLODEX_SECURE = True

#################
# Custom Config #
#################

from django.apps import AppConfig


class RolodexConfig(AppConfig):
	name = 'rolodex'
	verbose_name = 'Rolodex'



from django.test import TestCase
from django.test import Client
from django.core.urlresolvers import reverse
from rolodex.models import *
from django_webtest import WebTest
from rolodex.views import *
import json

client = Client()


class RolodexModelsTestCase(TestCase):

	print("Starting models tests...")

	fixtures = ['rolodex_models_testdata.json']

	def setUp(self):
		self.o1 = Org.objects.get(slug="acme-corp")
		self.p1 = Person.objects.get(slug="john-doe")
		self.o2 = Org.objects.get(slug="ajax-corp")
		self.p2 = Person.objects.get(slug="jane-doe")

	def test_getter_methods(self):
		self.assertEqual(list(self.p1.get_relations()['orgs']), [self.o1])
		self.assertEqual(list(self.p1.get_employer()), [self.o1])
		self.assertEqual(list(self.o1.get_employees()), [self.p1])
		self.assertEqual(list(self.o1.get_employees_by_role('boss')), [self.p1])
		self.assertEqual(self.o1.get_relations_by_type('employment')['people'], [{'type': 'employment', 'relation': self.p1}])
		self.assertEqual(self.o1.get_relations_with_type()['people'], [{'type': 'employment', 'relation': self.p1}])
		self.assertEqual(self.p1.nx_graph().has_node(self.o1), True)
		print(" >> Passed getter methods")

	def test_create_relationships(self):
		self.p1.add_p2p(self.p2)
		self.assertEqual(list(self.p1.get_relations()['people']), [self.p2])

		self.o1.add_org2org(self.o2, **{'hierarchy': 'parent'})
		self.assertEqual(Org2Org.objects.get(from_ent=self.o2, to_ent=self.o1).hierarchy, 'child')

		self.o1.add_org2p(self.p2)
		self.assertEqual(list(self.p2.get_relations()['orgs']), [self.o1])
		print(" >> Passed relationship create methods")

	def test_delete_relationships(self):
		self.p1.remove_p2p(self.p2)
		self.assertEqual(list(self.p2.get_relations_with_type()['people']), [])

		self.o1.remove_org2org(self.o2)
		self.assertEqual(list(self.o2.get_relations()['orgs']), [])

		self.o1.remove_org2p(self.p2)
		self.assertEqual(list(self.p2.get_relations()['orgs']), [])
		print(" >> Passed relationship delete methods")


'''
This test is not passing right now. See Org2Org model save method.
'''
# def test_hierarchy_recursion(self):
# 	self.o1.add_org2org(self.o2)
# 	relation = Org2Org.objects.get(from_ent=self.o1,to_ent=self.o2)
# 	relation.hierarchy = 'parent'
# 	relation.save()
# 	self.assertEqual(Org2Org.objects.get(from_ent=self.o2,to_ent=self.o1).hierarchy,'child')
# 	print(" >> Passed hierarchy recursion")


class RolodexViewsTestCase(WebTest):

	print("Starting views tests...")

	fixtures = ['rolodex_views_testdata.json']

	def setUp(self):
		self.o1 = Org.objects.get(slug="acme-corp")
		self.p1 = Person.objects.get(slug="john-doe")
		self.o2 = Org.objects.get(slug="ajax-corp")
		self.p2 = Person.objects.get(slug="jane-doe")

	def test_home(self):
		response = client.get(reverse('rolodex_home'))
		self.assertEqual(response.status_code, 200)
		print(" >> Passed home")

	def test_search(self):
		response = client.get(reverse('rolodex_org', args=[self.o1.slug]))
		self.assertContains(response, self.o1.orgName, status_code=200)
		response = client.get(reverse('rolodex_person', args=[self.p1.slug]))
		self.assertContains(response, self.p1.lastName, status_code=200)
		print(" >> Passed entity pages")

	def test_create_edit(self):
		'''
		Add a person with a bad email address. Fix the email and check for successful resubmit.
		'''
		form = self.app.get(reverse('rolodex_new_person', args=[self.o2.slug])).form
		form['lastName'] = 'Rabbit'
		form['firstName'] = 'Roger'
		form['person_contact-0-contact'] = 'nobugsbunny'
		form['person_contact-0-type'] = 'email'
		print(form['lastName'].value)
		response = form.submit()
		self.assertContains(response, 'Enter a valid email address.')
		form = response.form
		form['person_contact-0-contact'] = 'nobugs@gmail.com'
		response = form.submit()
		self.assertEqual(response.context['success'], True)
		'''
		Edit Person
		'''
		form = self.app.get(reverse('rolodex_edit_person', args=[self.p2.slug])).form
		form['lastName'] = 'Doe, Jr.'
		form['position'] = 'Scientist'
		response = form.submit().follow()
		self.assertContains(response, 'Doe, Jr.', status_code=200)
		'''
		Edit Org
		'''
		form = self.app.get(reverse('rolodex_edit_org', args=[self.o2.slug])).form
		form['orgName'] = 'AJAX, Inc.'
		response = form.submit().follow()
		self.assertContains(response, 'AJAX, Inc.', status_code=200)
		'''
		Edit Contact
		'''
		form = self.app.get(reverse('rolodex_edit_person', args=['roger-rabbit'])).form
		form['person_contact-0-contact'] = 'theRealRoger@gmail.com'
		response = form.submit().follow()
		self.assertContains(response, 'theRealRoger@gmail.com')
		# Delete the contact
		form = self.app.get(reverse('rolodex_edit_person', args=['roger-rabbit'])).form
		form['person_contact-0-DELETE'] = True
		response = form.submit().follow()
		self.assertNotContains(response, 'nobugs@gmail.com', status_code=200)
		print(" >> Passed entity create and edit pages")

	def test_remote_search(self):
		'''
		SELECT SEARCH FUNCTIONS

		entity_remote_search
		'''
		get_url = "%s?q=%s" % (reverse(entity_remote_search), 'ACME')
		response = client.get(get_url)
		expected_dict = [
			{
				'url': reverse(search_org, args=[self.o1.slug]),
				'name': self.o1.orgName
			}
		]
		self.assertContains(response, json.dumps(expected_dict), status_code=200)
		'''
		person_remote_search
		'''
		get_url = "%s?q=%s" % (reverse(person_remote_search), 'John')
		response = client.get(get_url)
		expected_dict = [
			{
				'pk': self.p1.pk,
				'p-url': reverse(search_person, args=[self.p1.slug]),
				'name': "%s, %s" % (self.p1.lastName, self.p1.firstName),
				'role': 'boss',
				'org': 'ACME, Corp.'
			}
		]
		self.assertContains(response, json.dumps(expected_dict), status_code=200)
		'''
		org_remote_search
		'''
		get_url = "%s?q=%s" % (reverse(org_remote_search), 'Acme')
		response = client.get(get_url)
		expected_dict = [
			{
				'pk': self.o1.pk,
				'org-url': reverse(search_org, args=[self.o1.slug]),
				'new-p-url': reverse(new_person, args=[self.o1.slug]),
				'name': "ACME, Corp.",
			}
		]
		self.assertContains(response, json.dumps(expected_dict), status_code=200)
		print(" >> Passed remote search test")

	def test_relation_create(self):
		'''
		Correspond to the new_org_relation & new_person_relation views.

		Org2Org
		'''
		form = self.app.get(reverse('rolodex_new_org_relation', args=[self.o1.slug])).forms[0]
		form['to_ent'].force_value(self.o2.pk)  # Force value for AJAX field
		form['hierarchy'] = 'parent'
		response = form.submit()
		self.assertEqual(response.status_code, 200)
		self.assertEqual(Org2Org.objects.get(from_ent=self.o2, to_ent=self.o1).hierarchy, 'child')
		'''
		Check no duplicate relationships
		'''
		form = self.app.get(reverse('rolodex_new_org_relation', args=[self.o1.slug])).forms[0]
		form['to_ent'].force_value(self.o2.pk)
		response = form.submit()
		self.assertContains(response, 'That relationship already exists.')

		'''
		Org2P
		'''
		form = self.app.get(reverse('rolodex_new_org_relation', args=[self.o2.slug])).forms[1]
		form['to_ent'].force_value(self.p2.pk)
		response = form.submit()
		self.assertEqual(response.context['saved'], True)
		'''
		P2P
		'''
		form = self.app.get(reverse('rolodex_new_person_relation', args=[self.p2.slug])).forms[1]
		form['to_ent'].force_value(self.p1.pk)
		response = form.submit()
		self.assertEqual(response.context['saved'], True)
		'''
		P2Org
		'''
		form = self.app.get(reverse('rolodex_new_person_relation', args=[self.p2.slug])).forms[0]
		form['to_ent'].force_value(self.o1.pk)
		response = form.submit()
		self.assertEqual(response.context['saved'], True)

		print(" >> Passed relationship create page")

	def test_delete(self):
		response = client.post(reverse('rolodex_delete_person', args=[self.p2.slug]))
		self.assertEqual(response.status_code, 302)
		response = client.post(reverse('rolodex_delete_org', args=[self.o2.slug]))
		self.assertEqual(response.status_code, 302)
		print(" >> Passed delete page")


'''
Fixtures...
'''
# from rolodex.models import *
# RELATION, get = P2Org_Type.objects.get_or_create(relationship_type='employment')
# ROLE = PersonRole.objects.create(role="boss")
# CONTACT_ROLE = OrgContactRole.objects.create(role="front desk")
# o1 = Org.objects.create(orgName="ACME, Corp.")
# p1 = Person.objects.create(firstName='John', lastName='Doe', role=ROLE)
# p1.add_p2org(o1,**{'relation':RELATION})
# Org.objects.create(orgName="AJAX, Corp.")
# Person.objects.create(firstName='Jane', lastName='Doe')

# python manage.py dumpdata rolodex --format=json --indent=4 > testproject/fixtures/rolodex_models_testdata.json
# python manage.py dumpdata rolodex --format=json --indent=4 > testproject/fixtures/rolodex_views_testdata.json



from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import redirect, render_to_response
from django.template import RequestContext
from django.http import HttpResponse, JsonResponse
from django.db.models import Q
from django.db import connection
from django.conf import settings
from django.core.urlresolvers import reverse
from django.views.decorators.csrf import csrf_exempt
from rolodex.models import Org, Person, Org2Org, P2Org_Type, P2P_Type, Org2Org_Type, SearchLog, Document
from rolodex.forms import OrgForm, OrgFormSet, PersonForm, PersonFormSet, P2PForm, Org2OrgForm, P2OrgForm, Org2PForm, DocumentForm
from datetime import date, timedelta
from itertools import chain
import json
from taggit.models import Tag
import networkx as nx

'''
Employment is the only required fixture.
'''

if 'rolodex_p2org' in connection.introspection.table_names():
	# using first right now to get around a testing bug where there seems to be a conflict on get between fixtures
	EMPLOYMENT = P2Org_Type.objects.filter(relationship_type='employment').first()
else:
	EMPLOYMENT = None


# timedelta for search activity calendar
minus_time = timedelta(days=-365 / 2)


def secure(view):
	'''
	Simple decorator that checks SETTINGS.
	'''
	if settings.ROLODEX_SECURE:
		return login_required(view)
	else:
		return view


@secure
def home(request):
	return render_to_response('rolodex/home.html', {}, context_instance=RequestContext(request))


############################################################################
# ORG Views #
#############

@secure
def new_org(request):
	form = OrgForm(instance=Org())
	formset = OrgFormSet(instance=Org())
	if request.method == "POST":
		form = OrgForm(request.POST)
		if form.is_valid():
			org = form.save()
			org.last_edited_by = user_check(request.user)
			org.save()
			formset = OrgFormSet(request.POST, instance=org)
			if formset.is_valid():
				formset.save()
				return redirect('rolodex_org', org.slug)
			else:
				org.delete()
				form = OrgForm(request.POST)
	return render_to_response('rolodex/new_org.html', {'form': form, 'formset': formset, }, context_instance=RequestContext(request))


@secure
def edit_org(request, org_slug):
	org_node = Org.objects.get(slug=org_slug)
	form = OrgForm(instance=org_node)
	formset = OrgFormSet(instance=org_node)
	formset.extra = 0
	if request.method == "POST":
		form = OrgForm(request.POST, instance=org_node)
		if form.is_valid():
			org = form.save()
			formset = OrgFormSet(request.POST, instance=org)
			if formset.is_valid():
				formset.save()
				org_node.last_edited_by = user_check(request.user)
				org_node.save()
				return redirect('rolodex_org', org.slug)

	return render_to_response('rolodex/new_org.html', {'form': form, 'formset': formset, 'orgNode': org_node, 'edit': True}, context_instance=RequestContext(request))


@permission_required('rolodex.delete_org')
def delete_org(request, org_slug):
	org = Org.objects.get(slug=org_slug)
	if request.method == "POST":
		org.delete()
		return redirect('rolodex_home')
	return render_to_response('rolodex/delete.html', {'org': org}, context_instance=RequestContext(request))


@secure
def search_org(request, org_slug):
	node = Org.objects.get(slug=org_slug)
	node.employees = node.get_employees()
	node.contacts = node.org_contact.all()
	node.relations = node.get_relations_with_type()
	node.net_length = len(net_compiler(Org.objects.filter(slug=org_slug), 3))
	# log the search
	logger(user=request.user, org=node)
	searches = SearchLog.objects.filter(org=node, datestamp__gt=date.today() - timedelta(days=365)).order_by('-datestamp')
	node.searches = [{'date': s.datestamp.strftime("%Y-%m-%d"), 'user': s.user} for s in searches]
	node.calendar = json.dumps(node.searches)
	node.documents = Document.objects.filter(org=node)
	node.tags = node.tags.all()
	tags = Tag.objects.all()
	return render_to_response('rolodex/org.html', {'node': node, 'tags': tags, }, context_instance=RequestContext(request))


@secure
def org_map(request, org_slug):
	node = Org.objects.get(slug=org_slug)
	hops = int(request.GET.get('hops', 3))
	return render_to_response('rolodex/orgMap.html', {'node': node, 'hops': hops}, context_instance=RequestContext(request))


@secure
def org_network(request, org_slug):
	network = net_compiler(Org.objects.filter(slug=org_slug), 3)
	data = json.dumps(network)
	return HttpResponse(data, content_type='application/json')


@secure
def adv_org_network(request, org_slug):
	hops = int(request.GET.get('hops', 3))
	network = net_compiler(Org.objects.filter(slug=org_slug), hops)
	org = Org.objects.get(slug=org_slug)
	centrality_data = adv_compile(org, hops)
	data = json.dumps({'centrality': centrality_data, 'links': network})
	return HttpResponse(data, content_type='application/json')


@secure
def new_org_relation(request, org_slug):
	saved = False
	org2org_form = Org2OrgForm()
	org2p_form = Org2PForm()
	if request.method == "POST":
		if request.POST['formType'] == '2P':
			org2p_form = Org2PForm(request.POST)
			if org2p_form.is_valid():
				from_ent = Org.objects.get(pk=request.POST['from_ent'])
				to_ent = Person.objects.get(pk=request.POST['to_ent'])
				relation = P2Org_Type.objects.get_or_none(relationship_type=request.POST['relation'],)
				from_ent.add_org2p(to_ent, **{'relation': relation})
				saved = True
		else:
			org2org_form = Org2OrgForm(request.POST)
			if org2org_form.is_valid():
				from_ent = Org.objects.get(pk=request.POST['from_ent'])
				to_ent = Org.objects.get(pk=request.POST['to_ent'])
				hierarchy = request.POST['hierarchy']
				relation = Org2Org_Type.objects.get_or_none(relationship_type=request.POST['relation'])
				from_ent.add_org2org(to_ent, **{'relation': relation, 'hierarchy': hierarchy})
				saved = True
	org_node = Org.objects.get(slug=org_slug)
	return render_to_response('rolodex/new_relation.html', {'orgNode': org_node, 'pForm': org2p_form, 'orgForm': org2org_form, 'saved': saved}, context_instance=RequestContext(request))


@secure
def new_org_doc(request, org_slug=None):
	if request.method == "POST":
		doc_form = DocumentForm(request.POST, request.FILES)
		if doc_form.is_valid():
			doc_form.save()
			return redirect('rolodex_org', org_slug)
		else:
			node = Person.objects.get(pk=request.POST['org'])
	else:
		node = Org.objects.get(slug=org_slug)
		doc_form = DocumentForm(initial={'org': node})
	return render_to_response('rolodex/new_doc.html', {'node': node, 'entity': 'org', 'docForm': doc_form}, context_instance=RequestContext(request))


############################################################################
# PERSON Views #
################

@secure
def new_person(request, org_slug=None):
	form = PersonForm()
	formset = PersonFormSet(instance=Person())
	success = False
	if request.method == "POST":
		form = PersonForm(request.POST)
		if form.is_valid():
			person = form.save()
			person.last_edited_by = user_check(request.user)
			formset = PersonFormSet(request.POST, instance=person)
			if formset.is_valid():
				formset.save()

				# Add employment relationship
				org_node = Org.objects.get(slug=org_slug)
				person.add_p2org(org_node, **{'relation': EMPLOYMENT})
				person.save()

				form = PersonForm()
				formset = PersonFormSet(instance=Person())
				success = True
			else:
				person.delete()
				form = PersonForm(request.POST)

	primary = Org.objects.get(slug=org_slug)
	return render_to_response('rolodex/new_person.html', {'form': form, 'formset': formset, 'orgNode': org_slug, 'primary': primary, 'success': success}, context_instance=RequestContext(request))


@secure
def new_person_no_org(request, org_slug=None):
	form = PersonForm()
	formset = PersonFormSet(instance=Person())
	success = False
	if request.method == "POST":
		form = PersonForm(request.POST)
		if form.is_valid():
			person = form.save()
			person.last_edited_by = user_check(request.user)
			formset = PersonFormSet(request.POST, instance=person)
			if formset.is_valid():
				formset.save()

				person.save()

				form = PersonForm()
				formset = PersonFormSet(instance=Person())
				success = True
			else:
				person.delete()
				form = PersonForm(request.POST)
	return render_to_response('rolodex/new_person.html', {'form': form, 'formset': formset, 'orgNode': org_slug, 'primary': None, 'success': success}, context_instance=RequestContext(request))


@secure
def edit_person(request, person_slug):
	peep = Person.objects.get(slug=person_slug)
	form = PersonForm(instance=peep)
	formset = PersonFormSet(instance=peep)
	formset.extra = 0
	if request.method == "POST":
		form = PersonForm(request.POST, instance=peep)

		if form.is_valid():
			person = form.save()
			formset = PersonFormSet(request.POST, instance=person)
			if formset.is_valid():
				formset.save()
				person.last_edited_by = user_check(request.user)
				person.save()
				return redirect('rolodex_person', person.slug)

	employer = peep.org_from_p.filter(relation=EMPLOYMENT)
	if len(employer) > 0:
		primary = employer[0].to_ent
	else:
		primary = "N/A"
	return render_to_response('rolodex/new_person.html', {'form': form, 'formset': formset, 'personNode': peep, 'primary': primary, 'edit': True}, context_instance=RequestContext(request))


@permission_required('rolodex.delete_person')
def delete_person(request, person_slug):
	peep = Person.objects.get(slug=person_slug)
	if request.method == "POST":
		peep.delete()
		return redirect('rolodex_home')
	return render_to_response('rolodex/delete.html', {'peep': peep}, context_instance=RequestContext(request))


@secure
def search_person(request, person_slug):
	node = Person.objects.get(slug=person_slug)
	employer = node.org_from_p.filter(relation=EMPLOYMENT)
	if len(employer) > 0:
		node.primary = employer[0].to_ent.orgName
	else:
		node.primary = "N/A"
	node.contacts = node.person_contact.all()
	node.relations = node.get_relations_with_type()
	node.net_length = len(net_compiler(Person.objects.filter(slug=person_slug), 3))
	# log the search
	logger(user=request.user, person=node)
	searches = SearchLog.objects.filter(person=node, datestamp__gt=date.today() - timedelta(days=365)).order_by('-datestamp')
	node.searches = [{'date': s.datestamp.strftime("%Y-%m-%d"), 'user': s.user} for s in searches]
	node.calendar = json.dumps(node.searches)
	node.documents = Document.objects.filter(person=node)
	node.tags = node.tags.all()
	tags = Tag.objects.all()
	return render_to_response('rolodex/person.html', {'node': node, 'tags': tags, }, context_instance=RequestContext(request))


@secure
def person_map(request, person_slug):
	node = Person.objects.get(slug=person_slug)
	hops = int(request.GET.get('hops', 3))
	return render_to_response('rolodex/personMap.html', {'node': node, 'hops': hops}, context_instance=RequestContext(request))


@secure
def person_network(request, person_slug):
	network = net_compiler(Person.objects.filter(slug=person_slug), 3)
	data = json.dumps(network)
	return HttpResponse(data, content_type='application/json')


@secure
def adv_person_network(request, person_slug):
	hops = int(request.GET.get('hops', 3))
	network = net_compiler(Person.objects.filter(slug=person_slug), hops)
	peep = Person.objects.get(slug=person_slug)
	centrality_data = adv_compile(peep, hops)
	data = json.dumps({'centrality': centrality_data, 'links': network})
	return HttpResponse(data, content_type='application/json')


@secure
def new_person_relation(request, person_slug):
	saved = False
	p2org_form = P2OrgForm()
	p2p_form = P2PForm()
	if request.method == "POST":
		if request.POST['formType'] == '2P':
			p2p_form = P2PForm(request.POST)
			if p2p_form.is_valid():
				'''
				We create the relationship via model method rather than the form
				in order to force the symetrical relationship created (so the
				get_relations method works).
				'''
				from_ent = Person.objects.get(pk=request.POST['from_ent'])
				to_ent = Person.objects.get(pk=request.POST['to_ent'])
				relation = P2P_Type.objects.get_or_none(relationship_type=request.POST['relation'])
				from_ent.add_p2p(to_ent, **{'relation': relation})
				saved = True
		else:
			p2org_form = P2OrgForm(request.POST)
			if p2org_form.is_valid():
				from_ent = Person.objects.get(pk=request.POST['from_ent'])
				to_ent = Org.objects.get(pk=request.POST['to_ent'])
				relation = P2Org_Type.objects.get_or_none(relationship_type=request.POST['relation'])
				from_ent.add_p2org(to_ent, **{'relation': relation})
				saved = True
	peep_node = Person.objects.get(slug=person_slug)
	return render_to_response('rolodex/new_relation.html', {'peepNode': peep_node, 'pForm': p2p_form, 'orgForm': p2org_form, 'saved': saved}, context_instance=RequestContext(request))


@secure
def new_person_doc(request, person_slug=None):
	if request.method == "POST":
		print(request.FILES)
		doc_form = DocumentForm(request.POST, request.FILES)
		if doc_form.is_valid():
			doc_form.save()
			return redirect('rolodex_person', person_slug)
		else:
			node = Person.objects.get(pk=request.POST['person'])
	else:
		node = Person.objects.get(slug=person_slug)
		doc_form = DocumentForm(initial={'person': node})
	return render_to_response('rolodex/new_doc.html', {'node': node, 'entity': 'person', 'docForm': doc_form}, context_instance=RequestContext(request))


@secure
def delete_relationship(request):
	'''
	Delete relationships via AJAX on entity pages.
	'''
	if request.POST:
		if request.POST['from_type'] == 'p':
			if request.POST['to_type'] == 'p':
				from_ent = Person.objects.get(pk=request.POST['from_ent'])
				to_ent = Person.objects.get(pk=request.POST['to_ent'])
				from_ent.remove_p2p(to_ent)
			else:
				from_ent = Person.objects.get(pk=request.POST['from_ent'])
				to_ent = Org.objects.get(pk=request.POST['to_ent'])
				from_ent.remove_p2org(to_ent)
		else:
			if request.POST['to_type'] == 'p':
				from_ent = Org.objects.get(pk=request.POST['from_ent'])
				to_ent = Person.objects.get(pk=request.POST['to_ent'])
				from_ent.remove_org2p(to_ent)
			else:
				from_ent = Org.objects.get(pk=request.POST['from_ent'])
				to_ent = Org.objects.get(pk=request.POST['to_ent'])
				from_ent.remove_org2org(to_ent)
	return HttpResponse("Done.")


@secure
def delete_doc(request):
	if request.POST:
		doc = Document.objects.get(pk=request.POST['id'])
		doc.delete()
		return HttpResponse("Done.")


@secure
def add_tag(request):
	if request.POST:
		if request.POST['entity'] == 'person':
			p = Person.objects.get(pk=request.POST['pk'])
			p.tags.add(request.POST['tag'])
		else:
			o = Org.objects.get(pk=request.POST['pk'])
			o.tags.add(request.POST['tag'])
		return HttpResponse("Done.")


@secure
def remove_tag(request):
	if request.POST:
		if request.POST['entity'] == 'person':
			p = Person.objects.get(pk=request.POST['pk'])
			p.tags.remove(request.POST['tag'])
		else:
			o = Org.objects.get(pk=request.POST['pk'])
			o.tags.remove(request.POST['tag'])
		return HttpResponse("Done.")


@secure
def search_tag(request, tag_name=None):
	tag = {}
	tag['name'] = tag_name
	tag['peeps'] = Person.objects.filter(tags__name=tag_name)
	tag['orgs'] = Org.objects.filter(tags__name=tag_name)
	tag['all_tags'] = Tag.objects.all()
	return render_to_response('rolodex/tag_search.html', {'tag': tag, }, context_instance=RequestContext(request))

#################################################################
# SELECT SEARCH FUNCTIONS #
###########################
# AJAX search for bloodhound/typeahead and selectize


@csrf_exempt
def entity_remote_search(request):
	q = request.GET.get('q', '')
	peeps = Person.objects.filter(Q(lastName__icontains=q) | Q(firstName__icontains=q))
	orgs = Org.objects.filter(orgName__icontains=q)
	response = [
		{
			'url': reverse(search_person, args=[p.slug]),
			'name': "%s, %s" % (p.lastName, p.firstName)
		} for p in peeps
	] + [
		{
			'url': reverse(search_org, args=[o.slug]),
			'name': o.orgName
		} for o in orgs
	]
	return JsonResponse(response, safe=False)


@csrf_exempt
def person_remote_search(request):
	q = request.GET.get('q', '')
	ignore = request.GET.get('ignore', None)  # On adding relations, we want to exclude the node itself
	peeps = Person.objects.filter(
		Q(lastName__icontains=q) |
		Q(firstName__icontains=q) |
		Q(role__role__icontains=q) |
		reduce(lambda x, y: x | y, [Q(orgs__orgName__contains=letter) for letter in q])
	).exclude(pk=ignore)  # Search firstName, lastName, role and org relations
	response = [
		{
			'pk': p.pk,
			'p-url': reverse(search_person, args=[p.slug]),
			'name': "%s, %s" % (p.lastName, p.firstName),
			'role': p.role.role if hasattr(p.role, 'role') else '',
			'org': p.orgName
		} for p in org_relate_peep(peeps)
	]
	return JsonResponse(response, safe=False)


@csrf_exempt
def org_remote_search(request):
	q = request.GET.get('q', '')
	ignore = request.GET.get('ignore', None)
	orgs = Org.objects.filter(orgName__icontains=q).exclude(pk=ignore)
	response = [
		{
			'pk': o.pk,
			'org-url': reverse(search_org, args=[o.slug]),
			'new-p-url': reverse(new_person, args=[o.slug]),
			'name': o.orgName
		} for o in orgs
	]
	return JsonResponse(response, safe=False)


#################################################################
# GRAPH HELPER FUNCTIONS #
##########################


def org_relate_peep(peeps):
	'''
	Helper to serialize data for search boxes.
	'''
	for peep in peeps:
		employers = peep.org_relations.filter(p_to_org__relation=EMPLOYMENT)
		if len(employers) > 0:
			peep.orgName = employers[0].orgName
			peep.orgID = employers[0].pk
		else:
			'''
			If person has no primary org relationship, or if it has been deleted, we list
			them with N/A for the purposes of the search boxes on home and relation page.
			'''
			peep.orgName = "N/A"
			peep.orgID = "N/A"
	return peeps


def hierarchy(snode, tnode):
	if snode.type == 'org' and tnode.type == 'org':
		snode.hierarchy = Org2Org.objects.get(from_ent=snode.pk, to_ent=tnode.pk).hierarchy
		tnode.hierarchy = Org2Org.objects.get(from_ent=tnode.pk, to_ent=snode.pk).hierarchy
	else:
		snode.hierarchy = 'none'
		tnode.hierarchy = 'none'
	return (snode, tnode)


def get_info(n):
	class Output:
		pass
	node = Output()
	if n.__class__.__name__ == 'Person':
		node.name = n.firstName + " " + n.lastName
		node.type = "person"
		node.id = "p" + str(n.slug)
	else:
		node.name = n.orgName
		node.type = "org"
		node.id = "o" + str(n.slug)
	node.pk = n.pk
	return node


def get_relations(nodes):
	relations, node_list = [], []
	for node in nodes:
		snode = get_info(node)
		relations = node.get_relations()
		for r in list(chain(relations['orgs'], relations['people'])):
			tnode = get_info(r)
			snode, tnode = hierarchy(snode, tnode)
			node_list.append({
				"source": snode.id,
				"source_name": str(snode.name),
				"source_type": str(snode.type),
				"source_hierarchy": str(snode.hierarchy),
				"target": tnode.id,
				"target_name": str(tnode.name),
				"target_type": str(tnode.type),
				"target_hierarchy": str(tnode.hierarchy),
			})
		relations = list(chain(relations['orgs'], relations['people']))
	relations = list(set(relations))
	return {'node_list': node_list, 'relations': relations}


def net_compiler(nodes, hops=2):
	node_list = []
	for i in range(hops):
		result = get_relations(nodes)
		nodes = result['relations']
		node_list += result['node_list']
	# de-dup
	node_list = [dict(t) for t in set([tuple(d.items()) for d in node_list])]
	return node_list


def adv_compile(node, hops):
	'''
	Compile some basic network centrality statistics for graph.
	'''
	graph = node.nx_graph(hops)
	degree = nx.degree_centrality(graph)
	betweenness = nx.betweenness_centrality(graph)
	closeness = nx.closeness_centrality(graph)
	data = {}
	for node in degree:
		n = get_info(node)
		data[n.id] = {
			'degree': degree[node],
			'betweenness': betweenness[node],
			'closeness': closeness[node],
		}
	return data


def user_check(user):
	if user.is_authenticated():
		return user.get_username()
	else:
		return "AnonymousUser"


def logger(user, person=None, org=None):
	user = user_check(user)
	if person:
		if not SearchLog.objects.filter(person=person, user=user, datestamp=date.today()).exists():
			SearchLog.objects.create(person=person, user=user)
	if org:
		if not SearchLog.objects.filter(org=org, user=user, datestamp=date.today()).exists():
			SearchLog.objects.create(org=org, user=user)



from django.conf.urls import url, include
from rolodex import views
from rolodex.API import router, views as api_views


urlpatterns = [
    url(r'^$', views.home, name='rolodex_home'),
    url(r'^org/(.+)/$', views.search_org, name='rolodex_org'),
    url(r'^person/(.+)/$', views.search_person, name='rolodex_person'),

    # Entity maintenance
    url(r'^add-person/(.+)/$', views.new_person, name='rolodex_new_person'),
    url(r'^add-person/$', views.new_person_no_org, name='rolodex_new_person_no_org'),
    url(r'^edit-person/(.+)/$', views.edit_person, name='rolodex_edit_person'),
    url(r'^delete-person/(.+)/$', views.delete_person, name='rolodex_delete_person'),
    url(r'^add-org/$', views.new_org, name='rolodex_new_org'),
    url(r'^edit-org/(.+)/$', views.edit_org, name='rolodex_edit_org'),
    url(r'^delete-org/(.+)/$', views.delete_org, name='rolodex_delete_org'),

    # Doc adds
    url(r'^add-doc/org/(.+)/$', views.new_org_doc, name='rolodex_new_org_doc'),
    url(r'^add-doc/person/(.+)/$', views.new_person_doc, name='rolodex_new_person_doc'),
    url(r'^delete-doc/$', views.delete_doc, name='rolodex_delete_doc'),

    # Relationship maintenance
    url(r'^add-relationship/org/(.+)/$', views.new_org_relation, name='rolodex_new_org_relation'),
    url(r'^add-relationship/person/(.+)/$', views.new_person_relation, name='rolodex_new_person_relation'),
    url(r'^delete-relationship/$', views.delete_relationship, name='rolodex_delete_relation'),

    url(r'^add-tag/$', views.add_tag, name='rolodex_add_tag'),
    url(r'^remove-tag/$', views.remove_tag, name='rolodex_remove_tag'),
    url(r'^search-tag/(.+)/$', views.search_tag, name='rolodex_search_tag'),

    # network graphs
    url(r'^person-map/(.+)/$', views.person_map, name='rolodex_person_map'),
    url(r'^org-map/(.+)/$', views.org_map, name='rolodex_org_map'),
    url(r'^person-network/(.+)/$', views.person_network, name='rolodex_person_network'),
    url(r'^org-network/(.+)/$', views.org_network, name='rolodex_org_network'),
    url(r'^person-network-advanced/person/(.+)/$', views.adv_person_network, name='rolodex_adv_person_network'),
    url(r'^org-network-advanced/org/(.+)/$', views.adv_org_network, name='rolodex_adv_org_network'),

    # bloodhound/selectize remote search
    url(r'^entity-remote/', views.entity_remote_search, name='rolodex_entity_remote_search'),
    url(r'^org-remote/', views.org_remote_search, name='rolodex_org_remote_search'),
    url(r'^person-remote/', views.person_remote_search, name='rolodex_person_remote_search'),

    # api
    url(r'^api/', include(router.router.urls)),
    url(r'^api/employees/([0-9]+)/$', api_views.GetEmployees.as_view())
]

'''
    Work on more API calls to match model methods (though some model funcs
        are passed strings and others objects...)
'''



from rolodex.models import Org, Person, P2P, Org2Org, P2Org, Org2P, Contact, Document
from django import forms
from django.forms import ModelForm
from django.forms.models import inlineformset_factory


class OrgForm(ModelForm):
    class Meta:
        model = Org
        fields = ['orgName', 'openRecordsLaw', 'notes']
        widgets = {
			'orgName': forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Name'}),
			'openRecordsLaw': forms.Select(attrs={'class': 'form-control'}),
			'notes': forms.Textarea(attrs={'class': 'form-control'}),
		}


class PersonForm(ModelForm):
	class Meta:
		model = Person
		fields = ['lastName', 'firstName', 'role', 'position', 'department', 'gender', 'birthdate', 'race', 'ethnicity', 'notes']
		widgets = {
			'lastName': forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Last name'}),
			'firstName': forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'First name'}),
			'position': forms.TextInput(attrs={'class': 'form-control'}),
			'department': forms.TextInput(attrs={'class': 'form-control'}),
			'gender': forms.Select(attrs={'class': 'form-control'}),
			'birthdate': forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Date/year of birth'}),
			'race': forms.Select(attrs={'class': 'form-control'}),
			'ethnicity': forms.Select(attrs={'class': 'form-control'}),
			'role': forms.Select(attrs={'class': 'form-control'}),
			'notes': forms.Textarea(attrs={'class': 'form-control'}),
		}

PersonFormSet = inlineformset_factory(
	Person,
	Contact,
	extra=1,
	can_delete=True,
	exclude=(),
	widgets = {
		'contact': forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Enter contact'}),
		'notes': forms.Textarea(attrs={'rows': 5, 'cols': 35, 'class': 'form-control', 'placeholder': 'Notes'}),
		'type': forms.Select(attrs={'class': 'form-control'}),
	})

OrgFormSet = inlineformset_factory(
	Org,
	Contact,
	extra=1,
	can_delete=True,
	exclude=(),
	widgets = {
		'contact': forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Enter contact'}),
		'notes': forms.Textarea(attrs={'rows': 5, 'cols': 35, 'class': 'form-control', 'placeholder': 'Notes'}),
		'type': forms.Select(attrs={'class': 'form-control'}),
		'role': forms.Select(attrs={'class': 'form-control role'}),
	})


class DocumentForm(ModelForm):
	class Meta:
		model = Document
		fields = '__all__'
		widgets = {
			'doc': forms.FileInput(attrs={'class': 'form-control'}),
			'link': forms.URLInput(attrs={'class': 'form-control'}),
			'notes': forms.Textarea(attrs={'rows': 5, 'cols': 35, 'class': 'form-control'}),
		}

######################
# Relationship Forms #
######################


class P2PForm(ModelForm):
	class Meta:
		model = P2P
		fields = ['from_ent', 'to_ent', 'relation', 'from_date', 'to_date', 'description']
		widgets = {'relation': forms.Select(attrs={'class': 'form-control'}), }


class P2OrgForm(ModelForm):
	class Meta:
		model = P2Org
		fields = ['from_ent', 'to_ent', 'relation', 'from_date', 'to_date', 'description']
		widgets = {'relation': forms.Select(attrs={'class': 'form-control'}), }


class Org2PForm(ModelForm):
	class Meta:
		model = Org2P
		fields = ['from_ent', 'to_ent', 'relation', 'from_date', 'to_date', 'description']
		widgets = {'relation': forms.Select(attrs={'class': 'form-control'}), }


class Org2OrgForm(ModelForm):
	class Meta:
		model = Org2Org
		fields = ['from_ent', 'to_ent', 'relation', 'from_date', 'to_date', 'hierarchy', 'description']
		widgets = {
			'relation': forms.Select(attrs={'class': 'form-control'}),
			'hierarchy': forms.Select(attrs={'class': 'form-control'})
		}



from rest_framework import serializers

from rolodex.models import Person, Org, Contact, PersonRole, OrgContactRole, P2P, Org2Org, P2Org, Org2P, P2P_Type, Org2Org_Type, P2Org_Type
from django.contrib.auth.models import User


class RoleSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = PersonRole
		fields = ('role', 'description')


class ContactRoleSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = OrgContactRole
		fields = ('role', 'description')


class P2P_TypeSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = P2P_Type
		fields = ('relationship_type',)


class Org2Org_TypeSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = Org2Org_Type
		fields = ('relationship_type',)


class P2Org_TypeSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = P2Org_Type
		fields = ('relationship_type',)


class PersonSerializer(serializers.HyperlinkedModelSerializer):
	# people = serializers.HyperlinkedRelatedField(many=True, read_only=True,view_name='p_relations')
	# orgs = serializers.HyperlinkedRelatedField(many=True, read_only=True,view_name='org_relations')
	# person_role = serializers.HyperlinkedRelatedField(many=True, read_only=False,view_name='role',queryset=PersonRole.objects.all())
	class Meta:
		model = Person
		fields = ('id', 'slug', 'firstName', 'lastName', 'position', 'department', 'gender', 'role', 'person_contact', 'org_relations', 'p_relations')


class OrgSerializer(serializers.HyperlinkedModelSerializer):
	# people = serializers.HyperlinkedRelatedField(many=True, read_only=True,view_name='person-relations')
	# orgs = serializers.HyperlinkedRelatedField(many=True, read_only=True,view_name='org-relations')
	class Meta:
		model = Org
		fields = ('id', 'slug', 'orgName', 'org_relations', 'p_relations', 'org_contact')


class ContactSerializer(serializers.HyperlinkedModelSerializer):
	# person_contact = serializers.HyperlinkedRelatedField( read_only=False,view_name='person',queryset=Person.objects.all())
	# org_contact = serializers.HyperlinkedRelatedField( read_only=False,view_name='org',queryset=Org.objects.all())
	class Meta:
		model = Contact
		fields = ('id', 'person', 'org', 'type', 'contact', 'role', 'notes')


class P2PSerializer(serializers.HyperlinkedModelSerializer):
	# p_from_p = serializers.HyperlinkedRelatedField( read_only=False,view_name='from_person',queryset=Person.objects.all())
	# p_to_p = serializers.HyperlinkedRelatedField( read_only=False,view_name='to_person',queryset=Person.objects.all())
	# p2p_relation = serializers.HyperlinkedRelatedField( read_only=False,view_name='relationship_type',queryset=P2P_Type.objects.all())
	class Meta:
		model = P2P
		fields = ('id', 'from_ent', 'to_ent', 'relation', 'from_date', 'to_date', 'description')


class Org2OrgSerializer(serializers.HyperlinkedModelSerializer):
	# org_from_org = serializers.HyperlinkedRelatedField( read_only=False,view_name='from_org',queryset=Org.objects.all())
	# org_to_org = serializers.HyperlinkedRelatedField( read_only=False,view_name='to_org',queryset=Org.objects.all())
	# org2org_relation = serializers.HyperlinkedRelatedField( read_only=False,view_name='relationship_type',queryset=Org2Org_Type.objects.all())
	class Meta:
		model = Org2Org
		fields = ('id', 'from_ent', 'to_ent', 'relation', 'from_date', 'to_date', 'description')


class P2OrgSerializer(serializers.HyperlinkedModelSerializer):
	# org_from_p = serializers.HyperlinkedRelatedField( read_only=False,view_name='from_person',queryset=Person.objects.all())
	# p_to_org = serializers.HyperlinkedRelatedField( read_only=False,view_name='to_org',queryset=Org.objects.all())
	# p2org_relation = serializers.HyperlinkedRelatedField( read_only=False,view_name='relationship_type',queryset=P2Org_Type.objects.all())
	class Meta:
		model = P2Org
		fields = ('id', 'from_ent', 'to_ent', 'relation', 'from_date', 'to_date', 'description')


class Org2PSerializer(serializers.HyperlinkedModelSerializer):
	# p_from_org = serializers.HyperlinkedRelatedField( read_only=False,view_name='from_org',queryset=Org.objects.all())
	# org_to_p = serializers.HyperlinkedRelatedField( read_only=False,view_name='to_person',queryset=Person.objects.all())
	# org2p_relation = serializers.HyperlinkedRelatedField( read_only=False,view_name='relationship_type',queryset=P2Org_Type.objects.all())
	class Meta:
		model = Org2P
		fields = ('id', 'from_ent', 'to_ent', 'relation', 'from_date', 'to_date', 'description')
